>From 8e95b4d112dc406ad76e7b056399dee555839db2 Mon Sep 17 00:00:00 2001
From: Sebastien RAILLARD <sr@coexsi.fr>
Date: Thu, 26 Mar 2015 13:46:54 +0100
Subject: [PATCH 6/6] Limit the number of CAM reset per output in order to limit the effect on other outputs

---
 config.h  |    3 ++-
 demux.c   |   10 +++++-----
 dvblast.c |    1 +
 dvblast.h |    2 ++
 en50221.c |    2 ++
 5 files changed, 12 insertions(+), 6 deletions(-)

diff --git a/config.h b/config.h
index 249cbf5..0a381a8 100644
--- a/config.h
+++ b/config.h
@@ -40,8 +40,9 @@
 #define DEFAULT_IPV6_MTU 1280
 #define PADDING_PID 8191
 #define WATCHDOG_WAIT 10000000LL
-#define WATCHDOG_REFRACTORY_PERIOD 60000000LL
+#define WATCHDOG_REFRACTORY_PERIOD 300000000LL
 #define MAX_ERRORS 1000
+#define MAX_RESET_PER_OUTPUT 3
 #define DEFAULT_VERBOSITY 4
 #define MAX_POLL_TIMEOUT 100000 /* 100 ms */
 #define MIN_POLL_TIMEOUT 100 /* 100 us */
diff --git a/demux.c b/demux.c
index 83e5336..d4b50cf 100644
--- a/demux.c
+++ b/demux.c
@@ -102,7 +102,6 @@ static mtime_t i_last_dts = -1;
 static int i_demux_fd;
 static int i_nb_errors = 0;
 static mtime_t i_last_error = 0;
-static mtime_t i_last_reset = 0;
 
 #ifdef HAVE_ICONV
 static iconv_t iconv_handle = (iconv_t)-1;
@@ -534,8 +533,10 @@ static void demux_Handle( block_t *p_ts )
                 else if ( i_wallclock > p_output->i_last_error + WATCHDOG_WAIT )
                     p_output->i_nb_errors = 0;
 
-                if ( p_output->i_nb_errors > MAX_ERRORS )
+                if ( p_output->i_nb_errors > MAX_ERRORS && p_output->i_nb_reset < MAX_RESET_PER_OUTPUT )
                 {
+                    p_output->i_nb_reset++;
+
                     int j;
                     for ( j = 0; j < i_nb_outputs; j++ )
                         pp_outputs[j]->i_nb_errors = 0;
@@ -546,10 +547,9 @@ static void demux_Handle( block_t *p_ts )
                         break;
                     default:
                         msg_Warn( NULL,
-                                 "too many errors for stream %s, resetting",
-                                 p_output->config.psz_displayname );
+                                 "Too many scrambled packets for stream %s - CAM reset #%d (max=%d)",
+                                 p_output->config.psz_displayname, p_output->i_nb_reset, MAX_RESET_PER_OUTPUT );
                     }
-                    i_last_reset = i_wallclock;
                     en50221_Reset();
                 }
             }
diff --git a/dvblast.c b/dvblast.c
index 8c20aed..3979c54 100644
--- a/dvblast.c
+++ b/dvblast.c
@@ -97,6 +97,7 @@ const char *psz_dvb_charset = "ISO-8859-1";
 const char *psz_provider_name = NULL;
 print_type_t i_print_type = PRINT_TEXT;
 bool b_print_enabled = false;
+mtime_t i_last_reset = 0;
 
 volatile sig_atomic_t b_conf_reload = 0;
 volatile sig_atomic_t b_exit_now = 0;
diff --git a/dvblast.h b/dvblast.h
index 7cd981d..25e0825 100644
--- a/dvblast.h
+++ b/dvblast.h
@@ -153,6 +153,7 @@ typedef struct output_t
 
     /* demux */
     int i_nb_errors;
+    int i_nb_reset;
     mtime_t i_last_error;
     uint8_t *p_pat_section;
     uint8_t i_pat_version, i_pat_cc;
@@ -236,6 +237,7 @@ extern const char *psz_native_charset;
 extern const char *psz_dvb_charset;
 extern enum print_type_t i_print_type;
 extern bool b_print_enabled;
+extern mtime_t i_last_reset;
 
 /* pid mapping */
 extern bool b_do_remap;
diff --git a/en50221.c b/en50221.c
index db35910..bebbc77 100644
--- a/en50221.c
+++ b/en50221.c
@@ -2012,6 +2012,8 @@ void en50221_Init( void )
  *****************************************************************************/
 void en50221_Reset( void )
 {
+    i_last_reset = i_wallclock;
+    
     memset( p_slots, 0, sizeof(ci_slot_t) * MAX_CI_SLOTS );
 
     if( i_ca_type & CA_CI_LINK )
-- 
1.7.0.4

